@@ -71,7 +71,7 @@ ctPlotArray <- function(input,
   plotcontrol=list(ylab='Array values',xaxs='i'),
   legend=TRUE,legendcontrol=list(),
   polygon=TRUE, polygonalpha=.1,polygoncontrol=list(steps=25)){
-  if(.Machine$sizeof.pointer == 4) message('Bayesian functions not available on 32 bit systems') else{  
+  if(!w32chk()) message('Bayesian functions not available on 32 bit systems') else{  
     if(class(input)!='list') stop('Input must be a list containing y and x subobjects!')
     
     x <- input$x
