@@ -1,4 +1,4 @@
-listOfMatrices <- function(df){
+listOfMatrices <- function(df,matnames=NA){
   mats <- unique(df$matrix)
   mlist <- list()
   for(mi in mats){
@@ -13,5 +13,11 @@ listOfMatrices <- function(df){
       }
     }
   }
+  
+  if(!is.na(matnames[1])){
+    for(mi in matnames){
+      mlist[[mi]] <- matrix(NA,0,0)
+    }
+  }
   return(mlist)
 }
